@@ -1,4 +1,4 @@
-/* $OpenBSD: p12_key.c,v 1.21 2014/07/12 16:03:37 miod Exp $ */
+/* $OpenBSD: p12_key.c,v 1.22 2015/02/07 13:19:15 doug Exp $ */
 /* Written by Dr Stephen N Henson (steve@openssl.org) for the OpenSSL
  * project 1999.
  */
@@ -89,7 +89,7 @@ PKCS12_key_gen_asc(const char *pass, int passlen, unsigned char *salt,
 	if (ret <= 0)
 		return 0;
 	if (unipass) {
-		OPENSSL_cleanse(unipass, uniplen);
+		explicit_bzero(unipass, uniplen);
 		free(unipass);
 	}
 	return ret;
