@@ -1,4 +1,4 @@
-/* $OpenBSD: x509_v3.c,v 1.11 2014/09/28 10:53:57 miod Exp $ */
+/* $OpenBSD: x509_v3.c,v 1.12 2015/07/29 14:58:34 jsing Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -248,7 +248,7 @@ X509_EXTENSION_set_object(X509_EXTENSION *ex, ASN1_OBJECT *obj)
 		return (0);
 	ASN1_OBJECT_free(ex->object);
 	ex->object = OBJ_dup(obj);
-	return (1);
+	return ex->object != NULL;
 }
 
 int
