@@ -1,4 +1,4 @@
-/* $OpenBSD: t1_lib.c,v 1.85 2015/09/12 16:10:08 doug Exp $ */
+/* $OpenBSD: t1_lib.c,v 1.86 2016/03/10 23:21:46 mmcc Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -774,7 +774,7 @@ ssl_add_clienthello_tlsext(SSL *s, unsigned char *p, unsigned char *limit)
 
 		/* NB: draft-ietf-tls-ecc-12.txt uses a one-byte prefix for
 		 * elliptic_curve_list, but the examples use two bytes.
-		 * http://www1.ietf.org/mail-archive/web/tls/current/msg00538.html
+		 * https://www1.ietf.org/mail-archive/web/tls/current/msg00538.html
 		 * resolves this to two bytes.
 		 */
 		s2n(curveslen * 2, ret);
@@ -1438,28 +1438,10 @@ ssl_parse_clienthello_tlsext(SSL *s, unsigned char **p, unsigned char *d,
 				/* Read in responder_id_list */
 				n2s(data, dsize);
 				size -= 2;
-				if (dsize > size) {
+				if (dsize > size  ) {
 					*al = SSL_AD_DECODE_ERROR;
 					return 0;
 				}
-
-				/*
-				 * We remove any OCSP_RESPIDs from a
-				 * previous handshake to prevent
-				 * unbounded memory growth.
-				 */
-				sk_OCSP_RESPID_pop_free(s->tlsext_ocsp_ids,
-				    OCSP_RESPID_free);
-				s->tlsext_ocsp_ids = NULL;
-				if (dsize > 0) {
-					s->tlsext_ocsp_ids =
-					    sk_OCSP_RESPID_new_null();
-					if (s->tlsext_ocsp_ids == NULL) {
-						*al = SSL_AD_INTERNAL_ERROR;
-						return 0;
-					}
-				}
-
 				while (dsize > 0) {
 					OCSP_RESPID *id;
 					int idsize;
@@ -1487,6 +1469,13 @@ ssl_parse_clienthello_tlsext(SSL *s, unsigned char **p, unsigned char *d,
 						*al = SSL_AD_DECODE_ERROR;
 						return 0;
 					}
+					if (!s->tlsext_ocsp_ids &&
+					    !(s->tlsext_ocsp_ids =
+					    sk_OCSP_RESPID_new_null())) {
+						OCSP_RESPID_free(id);
+						*al = SSL_AD_INTERNAL_ERROR;
+						return 0;
+					}
 					if (!sk_OCSP_RESPID_push(
 					    s->tlsext_ocsp_ids, id)) {
 						OCSP_RESPID_free(id);
@@ -2210,7 +2199,7 @@ tls_decrypt_ticket(SSL *s, const unsigned char *etick, int eticklen,
 		return -1;
 	}
 	EVP_DecryptUpdate(&ctx, sdec, &slen, p, eticklen);
-	if (EVP_DecryptFinal(&ctx, sdec + slen, &mlen) <= 0) {
+	if (EVP_DecryptFinal_ex(&ctx, sdec + slen, &mlen) <= 0) {
 		free(sdec);
 		EVP_CIPHER_CTX_cleanup(&ctx);
 		return 2;
